package com.ibm.bsf.engines.rexx;

import java.util.*;
import java.io.*;
import com.ibm.bsf.*;
import com.ibm.bsf.util.BSFEngineImpl;
import com.ibm.bsf.engines.rexx.RexxAndJava;


/**
  * This class manages the calls from Java via JNI to Rexx or Object Rexx
  * using the RexxSAA interface.
  *
  *
  *<pre>
  *  RexxEngine.java - this class makes (Object) Rexx available to IBM's BeanScriptingFramework (BSF)
  *  Copyright (C) 2001 Rony G. Flatscher
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
  *  License as published by the Free Software Foundation; either
  *  version 2.1 of the License, or (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *  The following URL may be available for further information:
  *
  *     <a href="http://www.gnu.org/copyleft/lesser.html">http://www.gnu.org/copyleft/lesser.html</a>
  *</pre>
  *
  * @version 1.0.4, 2001-05-12
  * @since 2001-03-01 by Rony G. Flatscher
  * @author Rony G. Flatscher (University of Essen &amp; WU Wien University)
  * @author Peter Kalender (Oct 2000 to Feb 2001, University of Essen)
  *
  */
public class RexxEngine extends BSFEngineImpl {

    /** Allows using the Rexx interpreter from Java. */
    private RexxAndJava   rexxInterface;

    /** Initializes the Rexx engine and causes the external
      * functions 'BSF' and 'BSFPollEvents' to be registered with the
      * Rexx interpreter. Therefore Rexx programs invoked from Java
      * have these two functions available to them and are therefore
      * able to communicate with Java.
      */
    public void initialize (BSFManager mgr, String lang,
                            Vector declaredBeans) throws BSFException
    {
        super.initialize (mgr, lang, declaredBeans);
        rexxInterface = new RexxAndJava(mgr, this);
        rexxInterface.jniRegisterBSF();
    }



    /** Evaluates (executes) a piece of Rexx code
      * (can be a whole program as well)
      * <em>without</em> any Java-supplied arguments and returns the result.
      *
      * @param source if set to {@link RexxAndJava#EVENT_TEXT},
      *               then <code>oscript</code> is put into the event queue
      *               to be polled by Rexx. If this text starts with the
      *               string <code>&quot;&#47;*2*&#47;&quot;</code> it is
      *               put into the alarm (highest dispatch) level vector, if it starts with the
      *               string <code>&quot;&#47;*0*&#47;&quot;</code> it is
      *               put into the batch (lowest dispatch) level vector,
      *               else it is put into the normal level vector.
      *
      * @param lineNo not used.
      * @param columnNo not used.
      * @param oscript the String version of this object [using toString()]
      *                represents the Rexx code to be executed.
      *                Its statements are delimited with a ';' or a
      *                CR (carriage-return) and/or LF (line-feed).
      *
      * @return a String object with the result from Rexx or <code>null</code>.
      */
    public Object eval (String source, int lineNo, int columnNo,Object oscript)
                                        throws BSFException
   {
        String script = oscript.toString ();

        if(source.equals(RexxAndJava.EVENT_TEXT))     // dispatching eventListener text
        {
           String tmp = script.substring(0, 5);

            if (tmp.equals("/*2*/"))                    // alarm (highest) level
            {
                rexxInterface.eventTextList.add(script, 2);
            }
            else if (tmp.equals("/*0*/"))               // batch (lowest) level
            {
                rexxInterface.eventTextList.add(script, 0);
            }
            else                                      // normal level
            {
                rexxInterface.eventTextList.add(script, 1);       // put text into normal level event  vector
            }

            return null;
        }
        else
        {
           Object obj=rexxInterface.jniRexxStart (script, null); // ---rony (now has a return value)
           return obj;
        }
    }



    /** Calls (executes) a Rexx program
      * (can be a single line of Rexx as well)
      * <em>with</em> Java-supplied arguments and returns the result.
      *
       @deprecated            &quot;

       This <code>call(...)</code> method may be enhanced in the future, once an openly
       defined and available interface to the OO-part of Object Rexx is made
       available by IBM. Then the argument &quot;<code>object</code>&quot; would denote
       an Object Rexx object, the argument &quot;<code>name</code>&quot; the name of one of its methods
       and the argument &quot;<code>args</code>&quot; the arguments for the method.


       <br> <strong>For the time being use method
    {@link #apply(String source, int lineNo, int columnNo, Object oscript, java.util.Vector vArgNames, java.util.Vector vArgs ) apply(...)}
       instead.</strong>
      *
      * @param object not used.
      * @param name at the moment will be taken as Rexx code to be executed.
      *                Rexx statements are delimited with a ';' or a
      *                CR (carriage-return) and/or LF (line-feed).
      *
      * @param args arguments to be passed to Rexx.
      *
      * @return a String object with the result from Rexx or <code>null</code>.
      */
    public Object call (Object object, String name, Object[] args)
                        throws BSFException {

        String stringArg[] = new String [ args.length ];
        for (int i=0 ; i < args.length ; i++)   // turn objects into strings for Rexx
        {
           if (args[i]!=null)
           {
              stringArg[i]=args[i].toString();
           }
           else stringArg[i]=null;
        }
System.err.println("RexxEngine.call(): do not use this method, use RexxEngine.apply(...) instead!"+
                   " This method will get enhanced, once the OO-interface to Object Rexx is made available by IBM.");

        return rexxInterface.jniRexxStart (name, stringArg);
    }




    /** Applies (evaluates) a piece of Rexx code
      * (can be a whole program as well)
      * <em>with</em> Java-supplied arguments and returns the result.
      *
      * @param source not used.
      * @param lineNo not used.
      * @param columnNo not used.
      * @param oscript the String version of this object [using toString()]
      *                represents the Rexx code to be executed.
      *                Its statements are delimited with a ';' or a
      *                an LF (line-feed, &quot;0x0a&quot;), preceeded by an
                       optional CR (carriage-return, &quot;0x0d&quot;).

      * @param vArgNames not used.
      * @param vArgs arguments to be passed to Rexx.
      *
      * @return a String object with the result from Rexx or <code>null</code>.
      */
    public Object apply (String source, int lineNo, int columnNo,
                         Object oscript,
                         java.util.Vector vArgNames,
                         java.util.Vector vArgs
                        )
                        throws BSFException {

        if (vArgs==null || vArgs.isEmpty())
        {
           return rexxInterface.jniRexxStart (oscript.toString(), null);
        }

        Object []oArr=vArgs.toArray();    // turn Vector into an array
        String stringArg[] = new String [ oArr.length ];
        for (int i=0; i<oArr.length; i++)
        {
           if (oArr[i] != null)
           {
              stringArg[i]=oArr[i].toString();
           }
           else stringArg[i]=null;
        }

        return rexxInterface.jniRexxStart (oscript.toString(), stringArg);
    }
}
